searchData={"items":[{"type":"module","title":"Boombox","doc":"Boombox is a tool for audio and video streaming.\n\nSee `run/1` for details and [examples.livemd](examples.livemd) for examples.","ref":"Boombox.html"},{"type":"function","title":"Boombox.run/2","doc":"Runs boombox with given input and output.","ref":"Boombox.html#run/2"},{"type":"function","title":"Example - Boombox.run/2","doc":"```\nBoombox.run(input: \"rtmp://localhost:5432\", output: \"index.m3u8\")\n```\n\nSee `t:input/0` and `t:output/0` for available outputs and [examples.livemd](examples.livemd)\nfor examples.\n\nIf the input is `{:stream, opts}`, a `Stream` or other `Enumerable` is expected\nas the first argument.","ref":"Boombox.html#run/2-example"},{"type":"function","title":"Boombox.run_cli/1","doc":"Runs boombox with CLI arguments.","ref":"Boombox.html#run_cli/1"},{"type":"function","title":"Example - Boombox.run_cli/1","doc":"```\n# boombox.exs\nMix.install([:boombox])\nBoombox.run_cli()\n```\n\n```sh\nelixir boombox.exs -i \"rtmp://localhost:5432\" -o \"index.m3u8\"\n```","ref":"Boombox.html#run_cli/1-example"},{"type":"type","title":"Boombox.in_stream_opts/0","doc":"","ref":"Boombox.html#t:in_stream_opts/0"},{"type":"type","title":"Boombox.input/0","doc":"","ref":"Boombox.html#t:input/0"},{"type":"type","title":"Boombox.out_stream_opts/0","doc":"","ref":"Boombox.html#t:out_stream_opts/0"},{"type":"type","title":"Boombox.output/0","doc":"","ref":"Boombox.html#t:output/0"},{"type":"type","title":"Boombox.webrtc_signaling/0","doc":"","ref":"Boombox.html#t:webrtc_signaling/0"},{"type":"module","title":"Boombox.Packet","doc":"Data structure emitted and accepted by `Boombox.run/1`\nwhen using `:stream` input or output.","ref":"Boombox.Packet.html"},{"type":"function","title":"Boombox.Packet.update_payload/2","doc":"","ref":"Boombox.Packet.html#update_payload/2"},{"type":"type","title":"Boombox.Packet.format/0","doc":"","ref":"Boombox.Packet.html#t:format/0"},{"type":"type","title":"Boombox.Packet.payload/0","doc":"","ref":"Boombox.Packet.html#t:payload/0"},{"type":"type","title":"Boombox.Packet.t/0","doc":"Data structure emitted and accepted by `Boombox.run/1`\nwhen using `:stream` input or output.","ref":"Boombox.Packet.html#t:t/0"},{"type":"extras","title":"Boombox","doc":"# Boombox\n\n[![Hex.pm](https://img.shields.io/hexpm/v/boombox.svg)](https://hex.pm/packages/boombox)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/boombox)\n[![CircleCI](https://circleci.com/gh/membraneframework/boombox.svg?style=svg)](https://circleci.com/gh/membraneframework/boombox)\n\nBoombox is a high-level tool for audio & video streaming tool based on the [Membrane Framework](https://membrane.stream).\n\nSee [examples.livemd](examples.livemd) for examples.","ref":"readme.html"},{"type":"extras","title":"Usage - Boombox","doc":"The code below receives a stream via RTMP and sends it over HLS:\n\n```elixir\nBoombox.run(input: \"rtmp://localhost:5432\", output: \"index.m3u8\")\n```\n\nyou can use CLI interface too:\n\n```sh\nboombox -i \"rtmp://localhost:5432\" -o \"index.m3u8\"\n```\n\nAnd the code below generates a video with bouncing Membrane logo and sends it over WebRTC:\n\n```elixir\nMix.install([:boombox, :req, :image])\n\noverlay =\n  Req.get!(\"https://avatars.githubusercontent.com/u/25247695?s=200&v=4\").body\n  |> Vix.Vips.Image.new_from_buffer()\n  |> then(fn {:ok, img} -> img end)\n  |> Image.trim!()\n  |> Image.thumbnail!(100)\n\nbg = Image.new!(640, 480, color: :light_gray)\nmax_x = Image.width(bg) - Image.width(overlay)\nmax_y = Image.height(bg) - Image.height(overlay)\n\nStream.iterate({_x = 300, _y = 0, _dx = 1, _dy = 2, _pts = 0}, fn {x, y, dx, dy, pts} ->\n  dx = if (x + dx) in 0..max_x, do: dx, else: -dx\n  dy = if (y + dy) in 0..max_y, do: dy, else: -dy\n  pts = pts + div(Membrane.Time.seconds(1), _fps = 60)\n  {x + dx, y + dy, dx, dy, pts}\nend)\n|> Stream.map(fn {x, y, _dx, _dy, pts} ->\n  img = Image.compose!(bg, overlay, x: x, y: y)\n  %Boombox.Packet{kind: :video, payload: img, pts: pts}\nend)\n|> Boombox.run(\n  input: {:stream, video: :image, audio: false},\n  output: {:webrtc, \"ws://localhost:8830\"}\n)\n```\n\nTo receive WebRTC/HLS from boombox in a browser or send WebRTC from a browser to boombox\nyou can use simple HTML examples in the `boombox_examples_data` folder, for example\n\n```sh\nwget https://raw.githubusercontent.com/membraneframework/boombox/v0.1.0/boombox_examples_data/webrtc_to_browser.html\nopen webrtc_to_browser.html\n```\n\nFor more examples, see [examples.livemd](examples.livemd).","ref":"readme.html#usage"},{"type":"extras","title":"Supported formats & protocols - Boombox","doc":"| format | direction |\n|---|---|\n| MP4 | input, output |\n| WebRTC | input, output |\n| RTMP | input |\n| RTSP | input |\n| HLS | output |\n| Elixir Stream | input, output |","ref":"readme.html#supported-formats-protocols"},{"type":"extras","title":"Installation - Boombox","doc":"To use Boombox as an Elixir library, add\n\n```elixir\n{:boombox, github: \"membraneframework-labs/boombox\"}\n```\n\nto your dependencies or `Mix.install`.\n\nto use via CLI, run the following:\n\n```sh\nwget https://raw.githubusercontent.com/membraneframework/boombox/v0.1.0/bin/boombox\nchmod u+x boombox\n./boombox\n```\n\nMake sure you have [Elixir](https://elixir-lang.org/) installed. The first call to `boombox` will install it in a default directory in the system. The directory can be set with `MIX_INSTALL_DIR` env variable if preferred.","ref":"readme.html#installation"},{"type":"extras","title":"CLI - Boombox","doc":"The CLI API is similar to the Elixir API, for example:\n\n```elixir\nBoombox.run(input: \"file.mp4\", output: {:webrtc, \"ws://localhost:8830\"})\n```\n\nis equivalent to:\n\n```sh\n./boombox -i file.mp4 -o --webrtc ws://localhost:8830\n```\n\nIt's also possible to pass an `.exs` script:\n\n```sh\n./boombox -s script.exs\n```\n\nIn the script you can call `Boombox.run(...)` and execute other Elixir code.\n\nThe first run of the CLI may take longer than usual, as the necessary artifacts are installed in the system.","ref":"readme.html#cli"},{"type":"extras","title":"Copyright and License - Boombox","doc":"Copyright 2024, [Software Mansion](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=boombox)\n\n[![Software Mansion](https://logo.swmansion.com/logo?color=white&variant=desktop&width=200&tag=membrane-github)](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=boombox)\n\nLicensed under the [Apache License, Version 2.0](LICENSE)","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Examples","doc":"# Boombox examples\n\n```elixir\nLogger.configure(level: :info)\n\n# For ffmpeg and ffplay commands to work on Mac Livebook Desktop\nSystem.put_env(\"PATH\", \"/opt/homebrew/bin:#{System.get_env(\"PATH\")}\")\n\n# In case of problems installing Nx/EXLA/Bumblebee,\n# you can remove them and the Nx backend config below.\n# Examples that don't mention them should still work.\nMix.install([:boombox, :kino, :nx, :exla, :bumblebee])\n\nNx.global_default_backend(EXLA.Backend)\n```","ref":"examples.html"},{"type":"extras","title":"Boombox - Examples","doc":"👋 Here are some examples of using Boombox. Some of them use [ffmpeg](https://www.ffmpeg.org/) to generate stream. Some use [ffplay](https://www.ffmpeg.org/ffplay.html) to playback generated videos, but you can use any other player, for example VLC.\n\nThe cell below downloads assets to be used in the examples, and runs an HTTP server on port `1234` that serves static HTML files for sending/receiving the stream in the browser.\n\n```elixir\ndata_dir = \"/tmp/boombox_examples_data\"\ninput_dir = \"#{data_dir}/input\"\nFile.mkdir_p!(input_dir)\nout_dir = \"#{data_dir}/output\"\nFile.mkdir_p!(out_dir)\n\nsamples_url =\n  \"https://raw.githubusercontent.com/membraneframework/static/gh-pages/samples\"\n\nunless File.exists?(\"#{input_dir}/bun.mp4\") do\n  %{status: 200, body: bbb_mp4} = Req.get!(\"#{samples_url}/big-buck-bunny/bun33s.mp4\")\n  File.write!(\"#{input_dir}/bun.mp4\", bbb_mp4)\nend\n\nassets_url =\n  \"https://raw.githubusercontent.com/membraneframework/boombox/master/boombox_examples_data\"\n\nfor asset <- [\"hls\", \"webrtc_from_browser\", \"webrtc_to_browser\"],\n    path = \"#{data_dir}/#{asset}.html\",\n    not File.exists?(path) do\n  %{status: 200, body: data} = Req.get!(\"#{assets_url}/#{asset}.html\")\n  File.write!(path, data)\nend\n\n# HTTP server for assets\n:inets.stop()\n:ok = :inets.start()\n\n{:ok, _server} =\n  :inets.start(:httpd,\n    bind_address: ~c\"localhost\",\n    port: 1234,\n    document_root: ~c\"#{data_dir}\",\n    server_name: ~c\"assets_server\",\n    server_root: ~c\"/tmp\",\n    erl_script_nocache: true\n  )\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#boombox"},{"type":"extras","title":"WebRTC proxy - Examples","doc":"Visit http://localhost:1234/webrtc_from_browser.html to send the stream and http://localhost:1234/webrtc_to_browser.html to receive it\n\n```elixir\nBoombox.run(input: {:webrtc, \"ws://localhost:8829\"}, output: {:webrtc, \"ws://localhost:8830\"})\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#webrtc-proxy"},{"type":"extras","title":"Record WebRTC to MP4 - Examples","doc":"To send the stream, visit http://localhost:1234/webrtc_from_browser.html.\n\nNote: don't stop this cell to finish recording - click 'disconnect' or close the browser tab instead, so the recording is finalized properly.\n\n```elixir\nBoombox.run(input: {:webrtc, \"ws://localhost:8829\"}, output: \"#{out_dir}/webrtc_to_mp4.mp4\")\n```\n\n```elixir\nSystem.shell(\"ffplay #{out_dir}/webrtc_to_mp4.mp4\")\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#record-webrtc-to-mp4"},{"type":"extras","title":"Broadcast MP4 via HLS - Examples","doc":"To receive the stream, visit http://localhost:1234/hls.html after running the cell below\n\n```elixir\nBoombox.run(input: \"#{input_dir}/bun.mp4\", output: {:hls, \"#{out_dir}/index.m3u8\"})\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#broadcast-mp4-via-hls"},{"type":"extras","title":"Broadcast RTMP via HLS - Examples","doc":"To receive the stream, visit http://localhost:1234/hls.html after running the cell below\n\n```elixir\nuri = \"rtmp://localhost:5432\"\n\nt =\n  Task.async(fn ->\n    Boombox.run(input: uri, output: \"#{out_dir}/index.m3u8\")\n  end)\n\n{_output, 0} = System.shell(\"ffmpeg -re -i #{input_dir}/bun.mp4 -c copy -f flv #{uri}\")\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#broadcast-rtmp-via-hls"},{"type":"extras","title":"Broadcast WebRTC via HLS - Examples","doc":"Visit http://localhost:1234/webrtc_from_browser.html to send the stream and http://localhost:1234/hls.html to receive it\n\n```elixir\nBoombox.run(input: {:webrtc, \"ws://localhost:8829\"}, output: {:hls, \"#{out_dir}/index.m3u8\"})\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#broadcast-webrtc-via-hls"},{"type":"extras","title":"Micro Twitch clone - Examples","doc":"```elixir\nKino.start_child!({\n  Membrane.RTMPServer,\n  handler: %Membrane.RTMP.Source.ClientHandlerImpl{controlling_process: self()},\n  port: 5001,\n  use_ssl?: false,\n  handle_new_client: fn client_ref, app, stream_key ->\n    hls_dir = \"#{out_dir}/#{stream_key}\"\n    File.mkdir_p!(hls_dir)\n\n    Task.start(fn ->\n      Boombox.run(input: {:rtmp, client_ref}, output: \"#{hls_dir}/index.m3u8\")\n    end)\n\n    Kino.Markdown.new(\"\"\"\n    New streamer connects with app #{app} and stream_key #{stream_key},\n    stream will be available at http://localhost:1234/hls.html?src=output/#{stream_key}/index.m3u8.\n    It may take a few seconds before the stream is playable.\n    \"\"\")\n    |> Kino.render()\n  end,\n  client_timeout: 1_000\n})\n\nbutton = Kino.Control.button(\"Connect streamer\")\nKino.render(button)\n\nbutton\n|> Stream.filter(fn event -> event.type == :click end)\n|> Kino.async_listen(fn _event ->\n  key = Base.encode16(:crypto.strong_rand_bytes(4))\n  uri = \"rtmp://localhost:5001/streamer/#{key}\"\n  {_output, 0} = System.shell(\"ffmpeg -re -i #{input_dir}/bun.mp4 -c copy -f flv #{uri}\")\nend)\n\n:ok\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#micro-twitch-clone"},{"type":"extras","title":"Read speech audio from MP4 chunk-by-chunk, generate transcription - Examples","doc":"```elixir\n{:ok, whisper} = Bumblebee.load_model({:hf, \"openai/whisper-tiny\"})\n{:ok, featurizer} = Bumblebee.load_featurizer({:hf, \"openai/whisper-tiny\"})\n{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"openai/whisper-tiny\"})\n{:ok, generation_config} = Bumblebee.load_generation_config({:hf, \"openai/whisper-tiny\"})\n\nserving =\n  Bumblebee.Audio.speech_to_text_whisper(\n    whisper,\n    featurizer,\n    tokenizer,\n    generation_config,\n    defn_options: [compiler: EXLA]\n  )\n\nBoombox.run(\n  input: \"#{samples_url}/sherlock_librivox.mp4\",\n  output:\n    {:stream,\n     video: false, audio: :binary, audio_rate: 16_000, audio_channels: 1, audio_format: :f32le}\n)\n|> Stream.map(&Nx.from_binary(&1.payload, :f32))\n|> Stream.chunk_every(200)\n|> Enum.each(fn chunk ->\n  batch = Nx.concatenate(chunk)\n\n  Nx.Serving.run(serving, batch).chunks\n  |> Enum.map_join(& &1.text)\n  |> IO.puts()\nend)\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#read-speech-audio-from-mp4-chunk-by-chunk-generate-transcription"},{"type":"extras","title":"Receive speech audio via WebRTC, generate live transcription - Examples","doc":"To send the stream, visit http://localhost:1234/webrtc_from_browser.html\n\n```elixir\n{:ok, whisper} = Bumblebee.load_model({:hf, \"openai/whisper-tiny\"})\n{:ok, featurizer} = Bumblebee.load_featurizer({:hf, \"openai/whisper-tiny\"})\n{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"openai/whisper-tiny\"})\n{:ok, generation_config} = Bumblebee.load_generation_config({:hf, \"openai/whisper-tiny\"})\n\nserving =\n  Bumblebee.Audio.speech_to_text_whisper(\n    whisper,\n    featurizer,\n    tokenizer,\n    generation_config,\n    defn_options: [compiler: EXLA]\n  )\n\nBoombox.run(\n  input: {:webrtc, \"ws://localhost:8829\"},\n  output:\n    {:stream,\n     video: false, audio: :binary, audio_rate: 16_000, audio_channels: 1, audio_format: :f32le}\n)\n|> Stream.map(&Nx.from_binary(&1.payload, :f32))\n|> Stream.chunk_every(200)\n|> Enum.each(fn chunk ->\n  batch = Nx.concatenate(chunk)\n\n  Nx.Serving.run(serving, batch).chunks\n  |> Enum.map_join(& &1.text)\n  |> IO.puts()\nend)\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#receive-speech-audio-via-webrtc-generate-live-transcription"},{"type":"extras","title":"Generate a bouncing logo video, stream it via WebRTC - Examples","doc":"To receive the stream, visit http://localhost:1234/webrtc_to_browser.html\n\n```elixir\noverlay =\n  Req.get!(\"https://avatars.githubusercontent.com/u/25247695?s=200&v=4\").body\n  |> Vix.Vips.Image.new_from_buffer()\n  |> then(fn {:ok, img} -> img end)\n  |> Image.trim!()\n  |> Image.thumbnail!(100)\n\nbg = Image.new!(640, 480, color: :light_gray)\nmax_x = Image.width(bg) - Image.width(overlay)\nmax_y = Image.height(bg) - Image.height(overlay)\n\nStream.iterate({_x = 300, _y = 0, _dx = 1, _dy = 2, _pts = 0}, fn {x, y, dx, dy, pts} ->\n  dx = if (x + dx) in 0..max_x, do: dx, else: -dx\n  dy = if (y + dy) in 0..max_y, do: dy, else: -dy\n  pts = pts + div(Membrane.Time.seconds(1), _fps = 60)\n  {x + dx, y + dy, dx, dy, pts}\nend)\n|> Stream.map(fn {x, y, _dx, _dy, pts} ->\n  img = Image.compose!(bg, overlay, x: x, y: y)\n  %Boombox.Packet{kind: :video, payload: img, pts: pts}\nend)\n|> Boombox.run(\n  input: {:stream, video: :image, audio: false},\n  output: {:webrtc, \"ws://localhost:8830\"}\n)\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#generate-a-bouncing-logo-video-stream-it-via-webrtc"},{"type":"extras","title":"Not hot dog - Examples","doc":"Inspired by Silicon Valley's [Not hot dog app](https://youtu.be/vIci3C4JkL0?si=jIa3Atu-QNgFld9h&t=60), and [Evadne Wu's talk](https://youtu.be/GFxUgbfQ3w0?si=7Y7kyid0GN_qffiB).\n\nTo send the stream, visit http://localhost:1234/webrtc_from_browser.html.\n\n```elixir\nmodel_name = \"google/vit-base-patch16-224\"\n{:ok, resnet} = Bumblebee.load_model({:hf, model_name})\n{:ok, featurizer} = Bumblebee.load_featurizer({:hf, model_name})\nserving = Bumblebee.Vision.image_classification(resnet, featurizer)\n:ok\n```\n\n```elixir\nframe = Kino.Frame.new()\nKino.render(frame)\n\nBoombox.run(\n  input: {:webrtc, \"ws://localhost:8829\"},\n  output: {:stream, video: :image, audio: false}\n)\n|> Stream.take_every(10)\n|> Stream.map(fn frame ->\n  tensor =\n    frame.payload\n    |> Image.thumbnail!(224)\n    |> Image.embed!(224, 224)\n    |> Image.to_nx!()\n\n  hot_dog =\n    Nx.Serving.run(serving, tensor).predictions\n    |> Enum.find(fn p -> String.contains?(p.label, \"hotdog\") end)\n\n  if hot_dog do\n    \"","ref":"examples.html#not-hot-dog"},{"type":"extras","title":"✅ Hotdog\" - Examples","doc":"else\n    \"","ref":"examples.html#hotdog"},{"type":"extras","title":"❌ Not hotdog\" - Examples","doc":"end\nend)\n|> Enum.each(fn text -> Kino.Frame.render(frame, Kino.Markdown.new(text)) end)\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#not-hotdog"},{"type":"extras","title":"Forward RTMP via WebRTC - Examples","doc":"To receive the stream, visit http://localhost:1234/webrtc_to_browser.html\n\n**Note**: due to a bug in Chrome, it may not work there unless launched with `--enable-features=WebRtcEncodedTransformDirectCallback`. See https://issues.chromium.org/issues/351275970.\n\n```elixir\nuri = \"rtmp://localhost:5434\"\n\nt =\n  Task.async(fn ->\n    Boombox.run(input: uri, output: {:webrtc, \"ws://localhost:8830\"})\n  end)\n\n{_output, 0} = System.shell(\"ffmpeg -re -i #{input_dir}/bun.mp4 -c copy -f flv #{uri}\")\nTask.await(t)\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#forward-rtmp-via-webrtc"},{"type":"extras","title":"Record RTMP to MP4 - Examples","doc":"```elixir\nuri = \"rtmp://localhost:5432\"\n\nt =\n  Task.async(fn ->\n    Boombox.run(input: uri, output: \"#{out_dir}/rtmp_to_mp4.mp4\")\n  end)\n\n{_output, 0} = System.shell(\"ffmpeg -re -i #{input_dir}/bun.mp4 -c copy -f flv #{uri}\")\n\nTask.await(t)\n```\n\n```elixir\nSystem.shell(\"ffplay #{out_dir}/rtmp_to_mp4.mp4\")\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#record-rtmp-to-mp4"},{"type":"extras","title":"Stream MP4 via WebRTC, receive it and record to MP4 again - Examples","doc":"```elixir\nsignaling = Membrane.WebRTC.SignalingChannel.new()\n\nt =\n  Task.async(fn ->\n    Boombox.run(input: \"#{input_dir}/bun.mp4\", output: {:webrtc, signaling})\n  end)\n\nBoombox.run(input: {:webrtc, signaling}, output: \"#{out_dir}/mp4_webrtc_mp4.mp4\")\n\nTask.await(t)\n```\n\n```elixir\nSystem.shell(\"ffplay #{out_dir}/mp4_webrtc_mp4.mp4\")\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#stream-mp4-via-webrtc-receive-it-and-record-to-mp4-again"},{"type":"extras","title":"Stream MP4 via WebRTC - Examples","doc":"To receive the stream, visit http://localhost:1234/webrtc_to_browser.html after running the cell below.\n\n**Note**: due to a bug in Chrome, it may not work there unless launched with `--enable-features=WebRtcEncodedTransformDirectCallback`. See https://issues.chromium.org/issues/351275970.\n\n```elixir\nBoombox.run(input: \"#{input_dir}/bun.mp4\", output: {:webrtc, \"ws://localhost:8830\"})\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#stream-mp4-via-webrtc"},{"type":"extras","title":"Stream MP4 via HTTP, forward it via WebRTC - Examples","doc":"To receive the stream, visit http://localhost:1234/webrtc_to_browser.html after running the cell below.\n\n**Note**: due to a bug in Chrome, it may not work there unless launched with `--enable-features=WebRtcEncodedTransformDirectCallback`. See https://issues.chromium.org/issues/351275970.\n\n```elixir\nBoombox.run(\n  input: \"#{samples_url}/big-buck-bunny/bun33s.mp4\",\n  output: {:webrtc, \"ws://localhost:8830\"}\n)\n```","ref":"examples.html#stream-mp4-via-http-forward-it-via-webrtc"},{"type":"extras","title":"License","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2020 Software Mansion\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}